import os

# See if Cython is installed
try:
    from Cython.Build import cythonize
# Do nothing if Cython is not available
except ImportError:
    # Got to provide this function. Otherwise, poetry will fail
    def build(setup_kwargs):
        pass
# Cython is installed. Compile
else:
    from setuptools import Extension
    from setuptools.dist import Distribution
    from distutils.command.build_ext import build_ext

    # This function will be executed in setup.py:
    def build(setup_kwargs):
        # The file you want to compile
        extensions = [
            "aiodeu/cetl.pyx"
        ]

        # gcc arguments hack: enable optimizations
        os.environ["CFLAGS"] = "-O3"

        # Build
        setup_kwargs.update({
            "ext_modules": cythonize(
                extensions,
                language_level=3,
                compiler_directives={"linetrace": True},
            ),
            "cmdclass": {"build_ext": build_ext}
        })



import csv
import io
from itertools import chain
from typing import Any, Union


def get_conditional_data(items: list, dot_path: str, values: Union[list, str, int]) -> Union[dict, None]:
    """
    Returns contitional data from within a dict

    :param items:
    :param dot_path:
    :param values:
    :return:
    """
    if not isinstance(values, list):
        values = [values]
    for item in items:
        item_value = get_field(dot_path, item)
        if item_value in values:
            return item
    return None


def get_field(dot_path: str, record: Any) -> Any:
    """
    Returns data in dict with dot path

    :param dot_path:
    :param record:
    :return:
    """
    field, *fields = dot_path.split(".")
    dp = ".".join(fields)
    if not field:
        if record is None:
            return ""
        return record
    if isinstance(record, dict):
        val = record.get(field)
        return get_field(dp, val)
    elif isinstance(record, list):
        if len(record):
            val = record[0].get(field)
            return get_field(dp, val)
        return [get_field(dp, val) for val in record]
    else:
        if record is None:
            return ""
        return record


def explode_lists(data: list, field: str) -> list:
    """
    Explode field within a dict of a list of dicts. Calls below function.

    :param data:
    :param field:
    :return:
    """
    return list(chain(*[explode_list(d, field) for d in data]))


def explode_list(data: dict, field: str, prefix: Union[str, None] = None) -> list:
    """
    Explode a dict in to a list of dicts on a particular field that is a nested list. This will duplicate top level
    fields.

    :param data:
    :param field:
    :param prefix:
    :return:
    """
    if prefix is None:
        prefix = f"{field}."
    data_list = []
    field_list = data[field]
    top_level_data = {key: value for key, value in data.items() if key != field}
    for row in field_list:
        d = {}
        d.update(top_level_data)
        d.update({f"{prefix}{key}": value for key, value in row.items()})
        data_list.append(d)
    return data_list


async def data_to_csv_string(data: list, fieldnames: list = []) -> str:
    """
    Convert response to csv string

    :param data:
    :param fieldnames:
    :return str:
    """
    output = io.StringIO()
    writer = csv.DictWriter(
        output,
        fieldnames or data[0].keys(),
        quoting=csv.QUOTE_ALL
    )
    writer.writeheader()
    writer.writerows(data)
    return output.getvalue()



from itertools import chain


cpdef get_field(str dot_path, record):
    """
    Returns data in dict with dot path

    :param dot_path:
    :param record:
    :return:
    """
    cdef str field
    cdef list fields
    field, *fields = dot_path.split(".")
    cdef str dp = ".".join(fields)
    if not field:
        if record is None:
            return ""
        return record
    if isinstance(record, dict):
        val = record.get(field)
        return get_field(dp, val)
    elif isinstance(record, list):
        if len(record):
            val = record[0].get(field)
            return get_field(dp, val)
        return [get_field(dp, val) for val in record]
    else:
        if record is None:
            return ""
        return record


cpdef get_conditional_data(list items, str dot_path, values):
    """
    Returns contitional data from within a dict

    :param items:
    :param dot_path:
    :param values:
    :return:
    """
    if not isinstance(values, list):
        values = [values]
    for item in items:
        item_value = get_field(dot_path, item)
        if item_value in values:
            return item
    return None


cpdef list explode_lists(list data, str field):
    """
    Explode field within a dict of a list of dicts. Calls below function.

    :param data:
    :param field:
    :return:
    """
    return list(chain(*[explode_list(d, field) for d in data]))


cpdef list explode_list(dict data, str field, str prefix = None):
    """
    Explode a dict in to a list of dicts on a particular field that is a nested list. This will duplicate top level
    fields.

    :param data:
    :param field:
    :param prefix:
    :return:
    """
    if prefix is None:
        prefix = "%s." % field
    cdef list data_list = []
    cdef list field_list = data[field]
    cdef dict top_level_data = {key: value for key, value in data.items() if key != field}
    cdef dict d
    for row in field_list:
        d = {}
        d.update(top_level_data)
        d.update({f"{prefix}{key}": value for key, value in row.items()})
        data_list.append(d)
    return data_list
